﻿Imports System.Drawing.Drawing2D

Public Class Form1
    ' Make some data.
    Private Sub Form1_Load(ByVal sender As System.Object, ByVal e As System.EventArgs) Handles MyBase.Load
        Dim list_views() As ListView = {lvwList, lvwSmallIcon, lvwLargeIcon, lvwTile, lvwDetails}

        For Each lvw As ListView In list_views
            AddItem(lvw, "Butterfly", My.Resources.Butterfly, Color.Green)
            AddItem(lvw, "Guppy", My.Resources.Fish, Color.Red)
            AddItem(lvw, "Peggy", My.Resources.Peggy, Color.Yellow)
        Next lvw
    End Sub

    ' Make a server status item.
    Private Sub AddItem(ByVal lvw As ListView, ByVal server As String, ByVal logo As Image, ByVal status As Color)
        ' Make the item.
        Dim item As New ListViewItem(server)

        ' Save the ServeStatus item in the Tag property.
        Dim server_status As New ServerStatus(server, logo, status)
        item.Tag = server_status
        item.SubItems(0).Name = "Server"

        ' Add subitems so they can draw.
        item.SubItems.Add("Logo")
        item.SubItems.Add("Status")

        ' Add the item to the ListView.
        lvw.Items.Add(item)
    End Sub

    ' Just draw the column's text.
    Private Sub lvw_DrawColumnHeader(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DrawListViewColumnHeaderEventArgs) Handles lvwTile.DrawColumnHeader, lvwSmallIcon.DrawColumnHeader, lvwList.DrawColumnHeader, lvwLargeIcon.DrawColumnHeader, lvwDetails.DrawColumnHeader
        Using string_format As New StringFormat()
            string_format.Alignment = StringAlignment.Center
            string_format.LineAlignment = StringAlignment.Center

            Dim text As String = lvwList.Columns(e.ColumnIndex).Text
            Select Case e.ColumnIndex
                Case 0
                    e.Graphics.DrawString(text, lvwList.Font, Brushes.Black, e.Bounds)
                Case 1
                    e.Graphics.DrawString(text, lvwList.Font, Brushes.Blue, e.Bounds)
                Case 2
                    e.Graphics.DrawString(text, lvwList.Font, Brushes.Green, e.Bounds)
            End Select
        End Using
    End Sub

    ' Draw the item. In this case, the server's logo.
    Private Sub lvw_DrawItem(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DrawListViewItemEventArgs) Handles lvwTile.DrawItem, lvwSmallIcon.DrawItem, lvwList.DrawItem, lvwLargeIcon.DrawItem, lvwDetails.DrawItem
        ' Draw Details view items in the DrawSubItem event handler.
        Dim lvw As ListView = e.Item.ListView
        If (lvw.View = View.Details) Then Return

        ' Get the ListView item and the ServerStatus object.
        Dim item As ListViewItem = e.Item
        Dim server_status As ServerStatus = DirectCast(item.Tag, ServerStatus)

        ' Clear.
        e.DrawBackground()

        ' Draw a status indicator.
        e.Graphics.SmoothingMode = SmoothingMode.AntiAlias
        Dim rect As New Rectangle( _
            e.Bounds.Left + 1, e.Bounds.Top + 1, _
            e.Bounds.Height - 2, e.Bounds.Height - 2)
        Using br As New SolidBrush(server_status.StatusColor)
            e.Graphics.FillEllipse(br, rect)
        End Using
        e.Graphics.DrawEllipse(Pens.Black, rect)
        Dim left As Integer = rect.Right + 2

        ' See how much we must scale it.
        Dim scale As Single = e.Bounds.Height / CSng(server_status.Logo.Height)

        ' Scale and position the image.
        e.Graphics.ScaleTransform(scale, scale)
        e.Graphics.TranslateTransform( _
            left, _
            e.Bounds.Top + (e.Bounds.Height - server_status.Logo.Height * scale) / 2, _
            System.Drawing.Drawing2D.MatrixOrder.Append)

        ' Draw the image.
        e.Graphics.DrawImage(server_status.Logo, 0, 0)

        ' Draw the focus rectangle if appropriate.
        e.Graphics.ResetTransform()
        e.DrawFocusRectangle()
    End Sub

    ' Draw subitems for Detail view.
    Private Sub lvw_DrawSubItem(ByVal sender As System.Object, ByVal e As System.Windows.Forms.DrawListViewSubItemEventArgs) Handles lvwTile.DrawSubItem, lvwSmallIcon.DrawSubItem, lvwList.DrawSubItem, lvwLargeIcon.DrawSubItem, lvwDetails.DrawSubItem
        ' Get the ListView item and the ServerStatus object.
        Dim item As ListViewItem = e.Item
        Dim server_status As ServerStatus = DirectCast(item.Tag, ServerStatus)

        ' Draw.
        Select Case e.ColumnIndex
            Case 0
                ' Draw the server's name.
                e.Graphics.DrawString(server_status.ServerName, _
                    lvwList.Font, Brushes.Black, e.Bounds)
            Case 1
                ' Draw the server's logo.
                Dim scale As Single = e.Bounds.Height / CSng(server_status.Logo.Height)
                e.Graphics.ScaleTransform(scale, scale)
                e.Graphics.TranslateTransform( _
                    e.Bounds.Left, _
                    e.Bounds.Top + (e.Bounds.Height - server_status.Logo.Height * scale) / 2, _
                    MatrixOrder.Append)
                e.Graphics.DrawImage(server_status.Logo, 0, 0)
            Case 2
                ' Draw the server's status.
                Dim rect As New Rectangle( _
                    e.Bounds.Left + 1, e.Bounds.Top + 1, _
                    e.Bounds.Width - 2, e.Bounds.Height - 2)
                Using br As New SolidBrush(server_status.StatusColor)
                    e.Graphics.FillRectangle(br, rect)
                End Using
                Dim pen_color As Color = Color.FromArgb(255, _
                    255 - server_status.StatusColor.R, _
                    255 - server_status.StatusColor.G, _
                    255 - server_status.StatusColor.B)
                Using br As New SolidBrush(pen_color)
                    Using string_format As New StringFormat()
                        string_format.Alignment = StringAlignment.Center
                        string_format.LineAlignment = StringAlignment.Center
                        Using the_font As New Font(lvwList.Font, FontStyle.Bold)
                            e.Graphics.DrawString(server_status.StatusColor.Name, _
                                Font, br, e.Bounds, string_format)
                        End Using
                    End Using
                End Using
        End Select

        ' Draw the focus rectangle if appropriate.
        e.Graphics.ResetTransform()
        Dim lvw As ListView = e.Item.ListView
        If (lvw.FullRowSelect) Then
            e.DrawFocusRectangle(e.Item.Bounds)
        ElseIf (e.SubItem.Name = "Server") Then
            e.DrawFocusRectangle(e.Bounds)
        End If
    End Sub
End Class
